Package: ggscribe
Title: Publication-Quality 'ggplot2' Annotation
Version: 0.1.1
Authors@R: 
    person(given = "David", family = "Hodge", 
           email = "davidhodge931@gmail.com", 
           role = c("aut", "cre", "cph"), 
           comment = c(ORCID = "0000-0002-3868-7501"))
Description: Annotation helper functions for publication-quality 'ggplot2' 
    visualisation. These functions make it easier to annotate plots in 
    a way that stays consistent with the set theme.
License: MIT + file LICENSE
URL: https://github.com/davidhodge931/ggscribe,
        https://davidhodge931.github.io/ggscribe/
BugReports: https://github.com/davidhodge931/ggscribe/issues
Depends: R (>= 4.1.0)
Imports: farver, ggplot2 (>= 4.0.0), glue, grid, rlang, scales
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Suggests: blends, dplyr, flexoki, ggrefine, ggwidth, jumble, knitr,
        rmarkdown, spelling, stringr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-04 03:01:52 UTC; david
Author: David Hodge [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3868-7501>)
Maintainer: David Hodge <davidhodge931@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 12:50:15 UTC
Built: R 4.5.2; ; 2026-05-04 17:01:13 UTC; unix

Package: ggscribe
Title: Publication-Quality 'ggplot2' Annotation
Version: 0.1.1
Authors@R: 
    person(given = "David", family = "Hodge", 
           email = "davidhodge931@gmail.com", 
           role = c("aut", "cre", "cph"), 
           comment = c(ORCID = "0000-0002-3868-7501"))
Description: Annotation helper functions for publication-quality 'ggplot2' 
    visualisation. These functions make it easier to annotate plots in 
    a way that stays consistent with the set theme.
License: MIT + file LICENSE
URL: https://github.com/davidhodge931/ggscribe,
        https://davidhodge931.github.io/ggscribe/
BugReports: https://github.com/davidhodge931/ggscribe/issues
Depends: R (>= 4.1.0)
Imports: farver, ggplot2 (>= 4.0.0), glue, grid, rlang, scales
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Suggests: blends, dplyr, flexoki, ggrefine, ggwidth, jumble, knitr,
        rmarkdown, spelling, stringr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-04 03:01:52 UTC; david
Author: David Hodge [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3868-7501>)
Maintainer: David Hodge <davidhodge931@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 12:50:15 UTC
Built: R 4.5.2; ; 2026-05-04 22:50:11 UTC; unix

# Generated by roxygen2: do not edit by hand

export(aes_contrast)
export(axis_bracket)
export(axis_line)
export(axis_text)
export(axis_ticks)
export(guide_sec_axis_text)
export(panel_shade)
export(reference_line)
export(sec_axis_text)
export(theme_sec_axis_text)
importFrom(rlang,":=")
importFrom(rlang,.data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ggscribe <a href="https://davidhodge931.github.io/ggscribe/"><img src="man/figures/logo.png" align="right" height="139" alt="ggscribe website" /></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/ggscribe)](https://CRAN.R-project.org/package=ggscribe)
<!-- badges: end -->

The objective of ggscribe is to provide annotation helper functions for
publication-quality ‘ggplot2’ visualisation.

Note:

- To avoid namespace collisions, it is recommended to not load the
  package, but instead refer to each function with the package name
  (e.g. `ggscribe::sec_axis_text()`.
- `sec_axis_text` adjusts space in the plot, whereas `axis_*` functions
  do not.
- `axis_ticks`, `axis_text` and `axis_bracket` require (1) a globally
  set theme with explicit panel dimensions and (2)
  `coord_cartesian(clip = "off")`
- `panel_shade` must be before geoms.
- Where you require annotation text along a axis with different angles
  etc, use a combination of `sec_axis_text` and `axis_*` functions. The
  `sec_axis_text` function should include the annotation that requires
  the maximum space that you want the plot to adjust to.

## Installation

Install from CRAN, or the development version from
[GitHub](https://github.com/davidhodge931/ggscribe).

``` r
install.packages("ggscribe")
pak::pak("davidhodge931/ggscribe")
```

## Example

ggscribe provides various axis and panel annotation helper functions.

``` r
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))
```

<img src="man/figures/README-unnamed-chunk-2-1.png" alt="" width="100%" />

And a function to ensure text is easily coloured for contrast on a fill
aesthetic.

``` r
ggwidth::set_equiwidth(equiwidth = 1.75)

mtcars |>
  count(cyl, am) |>
  mutate(
    am = if_else(am == 0, "Automatic", "Manual"),
    cyl = as.factor(cyl)
  ) |>
  ggplot(aes(x = am, y = n, colour = cyl, fill = cyl, label = n)) +
  geom_col(
    position = position_dodge2(preserve = "single", padding = 0.05),
    width = ggwidth::get_width(n = 2, n_dodge = 3),
  ) +
  scale_fill_discrete(palette = jumble::jumble) +
  scale_colour_discrete(palette = blends::multiply(jumble::jumble)) +
  geom_text(
    mapping = ggscribe::aes_contrast(), # or aes(!!!ggscribe::aes_contrast()),
    position = position_dodge2(
      width = ggwidth::get_width(n = 2, n_dodge = 3),
      padding = 0.05,
      preserve = "single"),
    vjust = 1.33,
    show.legend = FALSE,
  ) +
  scale_y_continuous(expand = expansion(c(0, 0.05))) +
  ggrefine::hybrid(x_type = "discrete")
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="" width="100%" />

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ggscribe <a href="https://davidhodge931.github.io/ggscribe/"><img src="man/figures/logo.png" align="right" height="139" alt="ggscribe website" /></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/ggscribe)](https://CRAN.R-project.org/package=ggscribe)
<!-- badges: end -->

The objective of ggscribe is to provide annotation helper functions for
publication-quality ‘ggplot2’ visualisation.

Note:

- To avoid namespace collisions, it is recommended to not load the
  package, but instead refer to each function with the package name
  (e.g. `ggscribe::sec_axis_text()`.
- `sec_axis_text` adjusts space in the plot, whereas `axis_*` functions
  do not.
- `axis_ticks`, `axis_text` and `axis_bracket` require (1) a globally
  set theme with explicit panel dimensions and (2)
  `coord_cartesian(clip = "off")`
- `panel_shade` must be before geoms.
- Where you require annotation text along a axis with different angles
  etc, use a combination of `sec_axis_text` and `axis_*` functions. The
  `sec_axis_text` function should include the annotation that requires
  the maximum space that you want the plot to adjust to.

## Installation

Install from CRAN, or the development version from
[GitHub](https://github.com/davidhodge931/ggscribe).

``` r
install.packages("ggscribe")
pak::pak("davidhodge931/ggscribe")
```

## Example

ggscribe provides various axis and panel annotation helper functions.

``` r
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))
```

<img src="man/figures/README-unnamed-chunk-2-1.png" alt="" width="100%" />

And a function to ensure text is easily coloured for contrast on a fill
aesthetic.

``` r
ggwidth::set_equiwidth(equiwidth = 1.75)

mtcars |>
  count(cyl, am) |>
  mutate(
    am = if_else(am == 0, "Automatic", "Manual"),
    cyl = as.factor(cyl)
  ) |>
  ggplot(aes(x = am, y = n, colour = cyl, fill = cyl, label = n)) +
  geom_col(
    position = position_dodge2(preserve = "single", padding = 0.05),
    width = ggwidth::get_width(n = 2, n_dodge = 3),
  ) +
  scale_fill_discrete(palette = jumble::jumble) +
  scale_colour_discrete(palette = blends::multiply(jumble::jumble)) +
  geom_text(
    mapping = ggscribe::aes_contrast(), # or aes(!!!ggscribe::aes_contrast()),
    position = position_dodge2(
      width = ggwidth::get_width(n = 2, n_dodge = 3),
      padding = 0.05,
      preserve = "single"),
    vjust = 1.33,
    show.legend = FALSE,
  ) +
  scale_y_continuous(expand = expansion(c(0, 0.05))) +
  ggrefine::modern(x_type = "discrete")
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="" width="100%" />

## Other packages

This package is part of a group of related packages built to extend
[ggplot2](https://ggplot2.tidyverse.org).

<table>

<tr>

<td align="center">

<a href="https://davidhodge931.github.io/ggblanket/"><img src="https://raw.githubusercontent.com/davidhodge931/ggblanket/main/man/figures/logo.svg" width="120" alt="ggblanket"/></a>
</td>

<td align="center">

<a href="https://davidhodge931.github.io/ggrefine/"><img src="https://raw.githubusercontent.com/davidhodge931/ggrefine/main/man/figures/logo.svg" width="120" alt="ggrefine"/></a>
</td>

<td align="center">

<a href="https://davidhodge931.github.io/ggscribe/"><img src="https://raw.githubusercontent.com/davidhodge931/ggscribe/main/man/figures/logo.svg" width="120" alt="ggscribe"/></a>
</td>

<td align="center">

<a href="https://davidhodge931.github.io/ggwidth/"><img src="https://raw.githubusercontent.com/davidhodge931/ggwidth/main/man/figures/logo.svg" width="120" alt="ggwidth"/></a>
</td>

<td align="center">

<a href="https://davidhodge931.github.io/blends/"><img src="https://raw.githubusercontent.com/davidhodge931/blends/main/man/figures/logo.svg" width="120" alt="blends"/></a>
</td>

<td align="center">

<a href="https://davidhodge931.github.io/jumble/"><img src="https://raw.githubusercontent.com/davidhodge931/jumble/main/man/figures/logo.svg" width="120" alt="jumble"/></a>
</td>

</tr>

</table>

#' A mapped aesthetic for text colour on fill
#'
#' @description Modifies a mapped colour (or fill) aesthetic for contrast against the fill (or colour) aesthetic.
#'
#' Function can be spliced into [ggplot2::aes] with [rlang::!!!].
#'
#' @param ... Require named arguments (and support trailing commas).
#' @param dark A dark colour. If NULL, derived from theme text or panel background.
#' @param light A light colour. If NULL, derived from theme text or panel background.
#' @param aesthetic The aesthetic to be modified for contrast. Either "colour" (default) or "fill".
#'
#' @return A ggplot2 aesthetic in [ggplot2::aes].
#'
#' @seealso
#' \code{\link[rlang]{splice}}
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' library(dplyr)
#' library(stringr)
#'
#' set_theme(
#'  ggrefine::theme_light(
#'     panel_heights = rep(unit(50, "mm"), 100),
#'     panel_widths = rep(unit(75, "mm"), 100),
#'  )
#' )
#'
#' ggwidth::set_equiwidth(equiwidth = 1.75)
#'
#' mtcars |>
#'   count(cyl, am) |>
#'   mutate(
#'     am = if_else(am == 0, "Automatic", "Manual"),
#'     cyl = as.factor(cyl)
#'   ) |>
#'   ggplot(aes(x = am, y = n, colour = cyl, fill = cyl, label = n)) +
#'   geom_col(
#'     position = position_dodge2(preserve = "single", padding = 0.05),
#'     width = ggwidth::get_width(n = 2, n_dodge = 3),
#'   ) +
#'   scale_fill_discrete(palette = jumble::jumble) +
#'   scale_colour_discrete(palette = blends::multiply(jumble::jumble)) +
#'   geom_text(
#'     mapping = ggscribe::aes_contrast(), # or aes(!!!ggscribe::aes_contrast()),
#'     position = position_dodge2(
#'       width = ggwidth::get_width(n = 2, n_dodge = 3),
#'       padding = 0.05,
#'       preserve = "single"),
#'     vjust = 1.33,
#'     show.legend = FALSE,
#'   ) +
#'   scale_y_continuous(expand = expansion(c(0, 0.05))) +
#'   ggrefine::modern(x_type = "discrete")
#'
#' mtcars |>
#'   count(cyl, am) |>
#'   mutate(
#'     am = if_else(am == 0, "automatic", "manual"),
#'     am = stringr::str_to_sentence(am),
#'     cyl = as.factor(cyl)
#'   ) |>
#'   ggplot(aes(y = am, x = n, colour = cyl, fill = cyl, label = n)) +
#'   geom_col(
#'     position = position_dodge2(preserve = "single", padding = 0.05),
#'     width = ggwidth::get_width(n = 2, n_dodge = 3, orientation = "y"),
#'   ) +
#'   scale_fill_discrete(palette = jumble::jumble) +
#'   scale_colour_discrete(palette = blends::multiply(jumble::jumble)) +
#'   geom_text(
#'     mapping = ggscribe::aes_contrast(), # or aes(!!!ggscribe::aes_contrast()),
#'     position = position_dodge2(
#'       width = ggwidth::get_width(n = 2, n_dodge = 3, orientation = "y"),
#'       preserve = "single",
#'       padding = 0.05,
#'     ),
#'     hjust = 1.25,
#'     show.legend = FALSE,
#'   ) +
#'   scale_x_continuous(expand = expansion(c(0, 0.05))) +
#'   ggrefine::modern(y_type = "discrete")
#'
aes_contrast <- function(..., dark = NULL, light = NULL, aesthetic = "colour") {
  # Only get theme if we need it
  if (rlang::is_null(dark) || rlang::is_null(light)) {
    # Get current theme
    current_theme <- ggplot2::get_theme()

    # Get text colour from theme
    theme_text <- current_theme$axis.text.x@colour %||%
      current_theme$axis.text.y@colour %||%
      current_theme$axis.text@colour %||%
      current_theme$text@colour %||%
      "black"

    # Get panel background from theme
    theme_panel <- current_theme$panel.background@fill %||%
      "white"

    # Determine which is dark and which is light using is_col_dark
    if (is_col_dark(theme_text)) {
      # Dark text theme (light mode)
      dark <- dark %||% theme_text
      light <- light %||% theme_panel
    } else {
      # Light text theme (dark mode)
      dark <- dark %||% theme_panel
      light <- light %||% theme_text
    }
  }

  if (aesthetic == "colour") {
    ggplot2::aes(
      colour = ggplot2::after_scale(
        get_contrast(col = .data$fill, dark = dark, light = light)
      )
    )
  } else if (aesthetic == "fill") {
    ggplot2::aes(
      fill = ggplot2::after_scale(
        get_contrast(col = .data$colour, dark = dark, light = light)
      )
    )
  } else {
    rlang::abort("aesthetic must be either 'colour' or 'fill'")
  }
}

# sec_axis_text --------------------------------------------------------------------

#' Secondary axis optimised for text annotations
#'
#' @param breaks One of:
#'    - `NULL` for no breaks
#'    - [ggplot2::waiver()] (default) to inherit breaks from the primary axis
#'    - A numeric vector of break positions
#'    - A function that takes the scale limits as input and returns break
#'      positions (e.g. `\(x) mean(c(x[2], 32))`)
#' @param name The name of the secondary axis. Use [ggplot2::waiver()] to
#'    derive the name from the primary axis, or `NULL` (default) for no name.
#' @param guide A guide object used to render the axis. Defaults to
#'    [guide_sec_axis_text()], which uses [theme_sec_axis_text()] to
#'    make transparent ticks and lines by default.
#' @param labels One of:
#'    - [ggplot2::derive()] (default) to derive labels from `breaks`
#'    - A character vector of labels, the same length as `breaks`
#'    - A function that takes break positions as input and returns labels
#' @param ... Additional arguments passed to [ggplot2::dup_axis()].
#'
#' @returns A `AxisSecondary` object for use in the `sec.axis` argument of
#'    `scale_x_continuous()` or `scale_y_continuous()`.
#'
#' @seealso [guide_sec_axis_text()], [theme_sec_axis_text()], [axis_text()]
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' library(dplyr)
#'
#' set_theme(
#'   ggrefine::theme_grey(
#'     panel_heights = rep(unit(50, "mm"), 100),
#'     panel_widths = rep(unit(75, "mm"), 100),
#'   )
#' )
#'
#' mtcars |>
#'   ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
#'   scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
#'   #clip = "off" is required for axis_text, axis_ticks and axis_bracket
#'   coord_cartesian(clip = "off") +
#'   #reference lines and shade
#'   ggscribe::reference_line(xintercept = 2.4) +
#'   ggscribe::reference_line(yintercept = 12)  +
#'   ggscribe::panel_shade(
#'     xmin = 4,
#'     xmax = 5,
#'   ) +
#'   #top axis
#'   scale_x_continuous(
#'     sec.axis = ggscribe::sec_axis_text(
#'       breaks = c(mean(c(4, 5))),
#'       labels = c("Range"),
#'       guide = ggscribe::guide_sec_axis_text(
#'         angle = 90,
#'       )
#'     )
#'   ) +
#'   ggscribe::axis_bracket(
#'     position = "top",
#'     breaks = c(4, 5),
#'   ) +
#'   ggscribe::axis_text(
#'     position = "top",
#'     breaks = c(2.4),
#'     labels = c("Threshold"),
#'   ) +
#'   #right axis
#'   ggscribe::axis_text(
#'     position = "right",
#'     breaks = 12,
#'     labels = "Threshold",
#'   ) +
#'   #'geom
#'   geom_point() +
#'   #annotations fit plot
#'   theme(plot.background = element_rect(colour = "grey92"))
#'
sec_axis_text <- function(
    breaks = ggplot2::waiver(),
    labels = ggplot2::derive(),
    name = NULL,
    guide = ggplot2::guide_axis(theme = theme_sec_axis_text()),
    ...
) {

  ggplot2::sec_axis(
    transform = "identity",
    breaks = breaks,
    labels = labels,
    name = name,
    guide = guide
  )
}

# axis_text -------------------------------------------------------------------

#' Annotate axis text
#'
#' Draws text labels at specified break positions along an axis, with style
#' defaults taken from the `axis.text` element of the set theme. Requires
#' `coord_cartesian(clip = "off")`.
#'
#' @param ... Not used. Forces named arguments.
#' @param position One of `"top"`, `"bottom"`, `"left"`, or `"right"`. Inferred
#'   from `xintercept` or `yintercept` if not provided.
#' @param xintercept For `"left"`/`"right"` axes: float the axis to this x
#'   position in data coordinates instead of the panel edge.
#' @param yintercept For `"top"`/`"bottom"` axes: float the axis to this y
#'   position in data coordinates instead of the panel edge.
#' @param breaks A numeric vector of break positions.
#' @param labels One of:
#'   - `NULL` (default) to use break values as labels
#'   - A character vector the same length as `breaks`
#'   - A function taking break values and returning labels
#' @param colour Inherits from `axis.text` in the set theme.
#' @param size Inherits from `axis.text` in the set theme.
#' @param family Inherits from `axis.text` in the set theme.
#' @param hjust,vjust Justification. Auto-calculated from `position` if `NULL`.
#' @param angle Text rotation angle. Defaults to `0`.
#' @param length Offset from the axis edge including tick length and margin.
#'   Supports `rel()`. Negative values place labels inside the panel. Defaults
#'   to `rel(1)` (theme tick length + text margin).
#'
#' @return A list of ggplot2 annotation layers.
#' @seealso [axis_line()], [axis_ticks()],
#'   [axis_bracket()], [reference_line()],
#'   [panel_shade()], [sec_axis_text()]
#' @export
#'
#' @inherit sec_axis_text examples
#'
axis_text <- function(
    ...,
    position     = NULL,
    xintercept   = NULL,
    yintercept   = NULL,
    breaks,
    labels       = NULL,
    colour       = NULL,
    size         = NULL,
    family       = NULL,
    hjust        = NULL,
    vjust        = NULL,
    angle        = 0,
    length = ggplot2::rel(1)
) {
  rlang::check_dots_empty()

  position <- .infer_position(position, xintercept, yintercept)
  axis     <- if (position %in% c("top", "bottom")) "x" else "y"

  .validate_intercept(axis, position, xintercept, yintercept)

  if (length(breaks) == 0) return(list())

  intercept     <- .resolve_intercept(axis, position, xintercept, yintercept)
  current_theme <- ggplot2::theme_get()

  text_hierarchy <- c(
    paste0("axis.text.", axis, ".", position),
    paste0("axis.text.", axis),
    "axis.text"
  )

  resolved_text_element <- NULL
  for (nm in text_hierarchy) {
    el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
    if (!is.null(el) && !inherits(el, "element_blank")) { resolved_text_element <- el; break }
  }
  if (is.null(resolved_text_element)) {
    resolved_text_element <- ggplot2::element_text(colour = "black", size = 11, family = "")
  }

  text_colour <- colour %||% resolved_text_element$colour %||% "black"
  text_size   <- size   %||% resolved_text_element$size   %||% 11
  text_family <- family %||% resolved_text_element$family %||% ""

  if (is.null(labels)) {
    labels <- as.character(breaks)
  } else if (is.function(labels)) {
    labels <- labels(breaks)
  }

  if (length(labels) != length(breaks)) {
    rlang::abort("Length of `labels` must match length of `breaks`.")
  }

  length_hierarchy <- c(
    paste0("axis.ticks.length.", axis, ".", position),
    paste0("axis.ticks.length.", axis),
    "axis.ticks.length"
  )
  resolved_length <- NULL
  for (nm in length_hierarchy) {
    el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
    if (!is.null(el) && !inherits(el, "element_blank")) { resolved_length <- el; break }
  }

  calculate_default_length <- function() {
    tl <- resolved_length
    if (is.null(tl)) {
      return(grid::unit(0.5 * (current_theme$text$size %||% 11), "pt"))
    } else if (inherits(tl, "rel")) {
      spacing_pts <- as.numeric(grid::convertUnit(current_theme$spacing %||% grid::unit(5.5, "pt"), "pt"))
      return(grid::unit(as.numeric(tl) * spacing_pts, "pt"))
    } else if (!inherits(tl, "unit")) {
      return(grid::unit(if (is.numeric(tl)) tl else 0.5 * (current_theme$text$size %||% 11), "pt"))
    } else {
      return(tl)
    }
  }

  flip_direction <- FALSE
  if (inherits(length, "rel")) {
    rel_value      <- as.numeric(length)
    default_pts    <- as.numeric(grid::convertUnit(calculate_default_length(), "pt"))
    length   <- grid::unit(abs(rel_value) * default_pts, "pt")
    flip_direction <- rel_value < 0
  } else if (inherits(length, "unit")) {
    tick_pts       <- as.numeric(grid::convertUnit(length, "pt"))
    length   <- grid::unit(abs(tick_pts), "pt")
    flip_direction <- tick_pts < 0
  } else if (is.numeric(length)) {
    flip_direction <- length < 0
    length   <- grid::unit(abs(length), "pt")
  }

  text_margin  <- resolved_text_element$margin
  margin_unit  <- grid::unit(2, "pt")
  if (!is.null(text_margin)) {
    margin_index <- switch(position, bottom = 1L, top = 3L, left = 2L, right = 4L)
    if (inherits(text_margin, c("margin", "unit")) && length(text_margin) >= margin_index) {
      margin_unit <- text_margin[margin_index]
    }
  }
  total_length <- length + margin_unit

  if (is.null(hjust)) {
    hjust <- if (position %in% c("top", "bottom")) 0.5
    else if (position == "left") { if (flip_direction) 0 else 1 }
    else { if (flip_direction) 1 else 0 }
  }
  if (is.null(vjust)) {
    vjust <- if (position == "bottom") { if (flip_direction) 0 else 1 }
    else if (position == "top") { if (flip_direction) 1 else 0 }
    else 0.5
  }

  make_gpar <- function() {
    grid::gpar(col = text_colour, fontsize = text_size, fontfamily = text_family)
  }

  lapply(seq_along(breaks), \(i) {
    break_val <- breaks[[i]]

    text_grob <- if (position == "bottom") {
      grid::textGrob(
        labels[i],
        x    = grid::unit(0.5, "npc"),
        y    = if (flip_direction) grid::unit(0, "npc") + total_length
        else                grid::unit(0, "npc") - total_length,
        just = c(hjust, vjust), rot = angle, gp = make_gpar()
      )
    } else if (position == "top") {
      grid::textGrob(
        labels[i],
        x    = grid::unit(0.5, "npc"),
        y    = if (flip_direction) grid::unit(1, "npc") - total_length
        else                grid::unit(1, "npc") + total_length,
        just = c(hjust, vjust), rot = angle, gp = make_gpar()
      )
    } else if (position == "left") {
      grid::textGrob(
        labels[i],
        x    = if (flip_direction) grid::unit(0, "npc") + total_length
        else                grid::unit(0, "npc") - total_length,
        y    = grid::unit(0.5, "npc"),
        just = c(hjust, vjust), rot = angle, gp = make_gpar()
      )
    } else {
      grid::textGrob(
        labels[i],
        x    = if (flip_direction) grid::unit(1, "npc") - total_length
        else                grid::unit(1, "npc") + total_length,
        y    = grid::unit(0.5, "npc"),
        just = c(hjust, vjust), rot = angle, gp = make_gpar()
      )
    }

    annotation_position <- if (axis == "x") {
      list(xmin = break_val, xmax = break_val, ymin = intercept, ymax = intercept)
    } else {
      list(xmin = intercept, xmax = intercept, ymin = break_val, ymax = break_val)
    }

    rlang::exec(ggplot2::annotation_custom, grob = text_grob, !!!annotation_position)
  })
}

# axis_bracket ----------------------------------------------------------------

#' Annotate an axis bracket
#'
#' Draws a bracket spanning `min(breaks)` to `max(breaks)` along an axis edge
#' or at a floating data position. The bar uses the same rendering path as
#' [axis_line()]; the caps use the same path as [axis_ticks()].
#' Requires `coord_cartesian(clip = "off")`.
#'
#' @param ... Not used. Forces named arguments.
#' @param position One of `"top"`, `"bottom"`, `"left"`, or `"right"`. Inferred
#'   from `xintercept` or `yintercept` if not provided.
#' @param xintercept For `"left"`/`"right"` axes: float the bracket to this x
#'   position in data coordinates instead of the panel edge.
#' @param yintercept For `"top"`/`"bottom"` axes: float the bracket to this y
#'   position in data coordinates instead of the panel edge.
#' @param breaks A numeric vector of length >= 2. The bar spans `min(breaks)`
#'   to `max(breaks)`; caps are drawn at every break value.
#' @param colour Inherits from `axis.ticks` in the set theme (falling back
#'   through `axis.line` and `line`).
#' @param linewidth Inherits from `axis.ticks` in the set theme. Supports `rel()`.
#' @param linetype Inherits from `axis.ticks` in the set theme.
#' @param length Length of the bracket caps as a grid unit. Supports
#'   `rel()`. Negative values flip the cap direction. Defaults to `rel(1)`
#'   (outward at theme tick length).
#'
#' @return A list of ggplot2 annotation layers.
#' @seealso [axis_line()], [axis_ticks()],
#'   [axis_text()], [reference_line()],
#'   [panel_shade()], [sec_axis_text()]
#' @export
#'
#' @inherit sec_axis_text examples
#'
axis_bracket <- function(
    ...,
    position     = NULL,
    xintercept   = NULL,
    yintercept   = NULL,
    breaks,
    colour       = NULL,
    linewidth    = NULL,
    linetype     = NULL,
    length = ggplot2::rel(1)
) {
  rlang::check_dots_empty()

  position <- .infer_position(position, xintercept, yintercept)
  axis     <- if (position %in% c("top", "bottom")) "x" else "y"

  .validate_intercept(axis, position, xintercept, yintercept)

  if (length(breaks) < 2) {
    rlang::abort("`breaks` must have at least 2 values to define the bracket span.")
  }

  bracket_from  <- min(breaks)
  bracket_to    <- max(breaks)
  intercept     <- .resolve_intercept(axis, position, xintercept, yintercept)
  current_theme <- ggplot2::theme_get()

  # ---- Resolve Style Element ------------------------------------------------
  # Walk axis.line → axis.ticks → line for consistent appearance.

  line_hierarchy <- c(
    paste0("axis.line.", axis, ".", position),
    paste0("axis.line.", axis),
    "axis.line",
    paste0("axis.ticks.", axis, ".", position),
    paste0("axis.ticks.", axis),
    "axis.ticks",
    "line"
  )
  resolved_element <- NULL
  for (nm in line_hierarchy) {
    el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
    if (!is.null(el) && !inherits(el, "element_blank")) { resolved_element <- el; break }
  }
  if (is.null(resolved_element)) {
    resolved_element <- list(colour = "#333333FF", linewidth = 0.5, linetype = 1)
  }

  line_colour    <- colour %||% resolved_element$colour %||% "#333333FF"
  line_linewidth <- if (is.null(linewidth)) {
    resolved_element$linewidth %||% 0.5
  } else if (inherits(linewidth, "rel")) {
    as.numeric(linewidth) * (resolved_element$linewidth %||% 0.5)
  } else {
    linewidth
  }
  line_linetype <- linetype %||% resolved_element$linetype %||% 1

  # ---- Resolve Cap Length ---------------------------------------------------

  length_hierarchy <- c(
    paste0("axis.ticks.length.", axis, ".", position),
    paste0("axis.ticks.length.", axis),
    "axis.ticks.length"
  )
  resolved_length <- NULL
  for (nm in length_hierarchy) {
    el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
    if (!is.null(el) && !inherits(el, "element_blank")) { resolved_length <- el; break }
  }

  calculate_theme_length <- function() {
    tl <- resolved_length
    if (is.null(tl)) {
      return(grid::unit(0.5 * (current_theme$text$size %||% 11), "pt"))
    } else if (inherits(tl, "rel")) {
      spacing_pts <- as.numeric(grid::convertUnit(current_theme$spacing %||% grid::unit(5.5, "pt"), "pt"))
      return(grid::unit(as.numeric(tl) * spacing_pts, "pt"))
    } else if (!inherits(tl, "unit")) {
      return(grid::unit(if (is.numeric(tl)) tl else 0.5 * (current_theme$text$size %||% 11), "pt"))
    } else {
      return(tl)
    }
  }

  flip_direction <- FALSE
  if (inherits(length, "rel")) {
    rel_value      <- as.numeric(length)
    default_pts    <- as.numeric(grid::convertUnit(calculate_theme_length(), "pt"))
    length   <- grid::unit(abs(rel_value) * default_pts, "pt")
    flip_direction <- rel_value < 0
  } else if (inherits(length, "unit")) {
    tick_pts       <- as.numeric(grid::convertUnit(length, "pt"))
    length   <- grid::unit(abs(tick_pts), "pt")
    flip_direction <- tick_pts < 0
  } else if (is.numeric(length)) {
    flip_direction <- length < 0
    length   <- grid::unit(abs(length), "pt")
  }

  # ---- Bar ------------------------------------------------------------------
  # Uses annotate("segment") — same rendering path as axis_line().

  stamp <- if (axis == "x") {
    list(ggplot2::annotate(
      "segment",
      x = bracket_from, xend = bracket_to,
      y = intercept,    yend = intercept,
      colour = line_colour, linewidth = line_linewidth, linetype = line_linetype
    ))
  } else {
    list(ggplot2::annotate(
      "segment",
      x = intercept,    xend = intercept,
      y = bracket_from, yend = bracket_to,
      colour = line_colour, linewidth = line_linewidth, linetype = line_linetype
    ))
  }

  # ---- Caps -----------------------------------------------------------------
  # annotation_custom + segmentsGrob pinned to each break — same rendering
  # path as axis_ticks(). Cap drawn at every break value.

  gp_cap <- ggplot2::gg_par(
    col = line_colour, stroke = line_linewidth, lty = line_linetype, lineend = "butt"
  )

  cap_grob <- if (position == "bottom") {
    grid::segmentsGrob(
      x0 = grid::unit(0.5, "npc"), x1 = grid::unit(0.5, "npc"),
      y0 = grid::unit(0, "npc"),
      y1 = if (flip_direction) grid::unit(0, "npc") + length
      else                grid::unit(0, "npc") - length,
      gp = gp_cap
    )
  } else if (position == "top") {
    grid::segmentsGrob(
      x0 = grid::unit(0.5, "npc"), x1 = grid::unit(0.5, "npc"),
      y0 = grid::unit(1, "npc"),
      y1 = if (flip_direction) grid::unit(1, "npc") - length
      else                grid::unit(1, "npc") + length,
      gp = gp_cap
    )
  } else if (position == "left") {
    grid::segmentsGrob(
      x0 = grid::unit(0, "npc"),
      x1 = if (flip_direction) grid::unit(0, "npc") + length
      else                grid::unit(0, "npc") - length,
      y0 = grid::unit(0.5, "npc"), y1 = grid::unit(0.5, "npc"),
      gp = gp_cap
    )
  } else {
    grid::segmentsGrob(
      x0 = grid::unit(1, "npc"),
      x1 = if (flip_direction) grid::unit(1, "npc") - length
      else                grid::unit(1, "npc") + length,
      y0 = grid::unit(0.5, "npc"), y1 = grid::unit(0.5, "npc"),
      gp = gp_cap
    )
  }

  cap_annotations <- lapply(breaks, \(break_val) {
    cap_pos <- if (axis == "x") {
      list(xmin = break_val, xmax = break_val, ymin = intercept, ymax = intercept)
    } else {
      list(xmin = intercept, xmax = intercept, ymin = break_val, ymax = break_val)
    }
    rlang::exec(ggplot2::annotation_custom, grob = cap_grob, !!!cap_pos)
  })

  c(stamp, cap_annotations)
}

# panel_shade -----------------------------------------------------------------

#' Annotate a shaded panel region
#'
#' Draws a filled rectangle over the panel with colour defaults taken from the
#' set theme. Defaults to a subtle overlay across the full panel, with the fill
#' automatically adapting to light or dark panel backgrounds. Should be placed
#' before geom layers.
#'
#' @param ... Not used. Allows trailing commas and named-argument style calls.
#' @param xmin,xmax Left and right edges of the rectangle. Defaults to `-Inf`
#'   and `Inf`. Use `I()` for normalized coordinates (0-1).
#' @param ymin,ymax Bottom and top edges of the rectangle. Defaults to `-Inf`
#'   and `Inf`. Use `I()` for normalized coordinates (0-1).
#' @param fill Fill colour. Defaults to a neutral grey.
#' @param alpha Opacity of the rectangle. Defaults to `0.25`.
#' @param colour Border colour. Defaults to `"transparent"`.
#' @param linewidth Inherits from `panel.border` in the set theme. Supports
#'   `rel()`.
#' @param linetype Border linetype. Defaults to `1`.
#'
#' @return A list containing an annotation layer.
#' @export
#'
#' @inherit sec_axis_text examples
#'
panel_shade <- function(
    ...,
    xmin = -Inf,
    xmax = Inf,
    ymin = -Inf,
    ymax = Inf,
    fill = "#878580",
    alpha = 0.25,
    colour = "transparent",
    linewidth = NULL,
    linetype = NULL
) {
  rlang::check_dots_empty()

  xmin_is_normalized <- inherits(xmin, "AsIs")
  xmax_is_normalized <- inherits(xmax, "AsIs")
  ymin_is_normalized <- inherits(ymin, "AsIs")
  ymax_is_normalized <- inherits(ymax, "AsIs")

  x_uses_normalized <- xmin_is_normalized || xmax_is_normalized
  y_uses_normalized <- ymin_is_normalized || ymax_is_normalized

  if (x_uses_normalized) {
    if (
      !((xmin_is_normalized || is.infinite(xmin)) &&
        (xmax_is_normalized || is.infinite(xmax)))
    ) {
      rlang::abort(
        "Cannot mix normalized (I()) and data coordinates for x. Use I() for both xmin and xmax, or neither."
      )
    }
  }
  if (y_uses_normalized) {
    if (
      !((ymin_is_normalized || is.infinite(ymin)) &&
        (ymax_is_normalized || is.infinite(ymax)))
    ) {
      rlang::abort(
        "Cannot mix normalized (I()) and data coordinates for y. Use I() for both ymin and ymax, or neither."
      )
    }
  }

  if (xmin_is_normalized) {
    xmin <- unclass(xmin)
    if (length(xmin) != 1 || xmin < 0 || xmin > 1) {
      rlang::abort("Normalized xmin must be a single value between 0 and 1.")
    }
  }
  if (xmax_is_normalized) {
    xmax <- unclass(xmax)
    if (length(xmax) != 1 || xmax < 0 || xmax > 1) {
      rlang::abort("Normalized xmax must be a single value between 0 and 1.")
    }
  }
  if (ymin_is_normalized) {
    ymin <- unclass(ymin)
    if (length(ymin) != 1 || ymin < 0 || ymin > 1) {
      rlang::abort("Normalized ymin must be a single value between 0 and 1.")
    }
  }
  if (ymax_is_normalized) {
    ymax <- unclass(ymax)
    if (length(ymax) != 1 || ymax < 0 || ymax > 1) {
      rlang::abort("Normalized ymax must be a single value between 0 and 1.")
    }
  }

  use_grob <- x_uses_normalized || y_uses_normalized

  current_theme <- ggplot2::theme_get()
  panel_border <- ggplot2::calc_element(
    "panel.border",
    current_theme,
    skip_blank = TRUE
  )
  base_linewidth <- if (
    !is.null(panel_border) && !inherits(panel_border, "element_blank")
  ) {
    panel_border$linewidth %||% 0.5
  } else {
    0.5
  }

  alpha <- alpha %||% 1
  linewidth <- if (is.null(linewidth)) {
    base_linewidth
  } else if (inherits(linewidth, "rel")) {
    as.numeric(linewidth) * base_linewidth
  } else {
    linewidth
  }
  linetype <- linetype %||% 1

  if (use_grob) {
    x_left <- if (xmin_is_normalized) {
      grid::unit(xmin, "npc")
    } else {
      grid::unit(0, "npc")
    }
    x_right <- if (xmax_is_normalized) {
      grid::unit(xmax, "npc")
    } else {
      grid::unit(1, "npc")
    }
    y_bottom <- if (ymin_is_normalized) {
      grid::unit(ymin, "npc")
    } else {
      grid::unit(0, "npc")
    }
    y_top <- if (ymax_is_normalized) {
      grid::unit(ymax, "npc")
    } else {
      grid::unit(1, "npc")
    }

    rect_grob <- grid::rectGrob(
      x = x_left,
      y = y_bottom,
      width = x_right - x_left,
      height = y_top - y_bottom,
      just = c("left", "bottom"),
      gp = ggplot2::gg_par(
        fill = scales::alpha(fill, alpha),
        col = colour,
        stroke = linewidth,
        lty = linetype
      )
    )

    list(ggplot2::annotation_custom(
      rect_grob,
      xmin = -Inf,
      xmax = Inf,
      ymin = -Inf,
      ymax = Inf
    ))
  } else {
    list(ggplot2::annotate(
      "rect",
      xmin = xmin,
      xmax = xmax,
      ymin = ymin,
      ymax = ymax,
      fill = fill,
      colour = colour,
      linewidth = linewidth,
      linetype = linetype,
      alpha = alpha
    ))
  }
}

# Internal helpers ------------------------------------------------------------

.validate_intercept <- function(axis, position, xintercept, yintercept) {
  if (!is.null(xintercept) && axis == "x") {
    rlang::abort(glue::glue(
      "`xintercept` is not applicable for position = \"{position}\". ",
      "Did you mean `yintercept`?"
    ))
  }
  if (!is.null(yintercept) && axis == "y") {
    rlang::abort(glue::glue(
      "`yintercept` is not applicable for position = \"{position}\". ",
      "Did you mean `xintercept`?"
    ))
  }
  invisible(NULL)
}

.resolve_intercept <- function(axis, position, xintercept, yintercept) {
  if (axis == "x") {
    yintercept %||% if (position == "bottom") -Inf else Inf
  } else {
    xintercept %||% if (position == "left") -Inf else Inf
  }
}

.infer_position <- function(position, xintercept, yintercept) {
  if (!is.null(position)) {
    return(rlang::arg_match(position, c("top", "bottom", "left", "right")))
  }
  if (!is.null(yintercept)) return("bottom")
  if (!is.null(xintercept)) return("left")
  rlang::abort(
    "Must specify `position`, or supply `xintercept` (implies left/right) or `yintercept` (implies top/bottom)."
  )
}

#' Get a dark/light colour for contrast
#'
#' @description Get a dark/light colour based on contrast.
#'
#' @param ... Require named arguments (and support trailing commas).
#' @param col A vector of colours from which to determine a contrast vector of light/dark colours.
#' @param dark A dark colour. If NULL, derived from theme text or panel background.
#' @param light A light colour. If NULL, derived from theme text or panel background.
#'
#' @return A character vector of colours, the same length as the `col` vector, containing either
#'         the dark or light colour determined for contrast.
#'
#' @noRd
#'
#' @examples
#' get_contrast(col = c("#000000", "#FFFFFF", "#808080"))  # Uses theme colours
#' get_contrast(col = c("navy", "yellow", "orange"), dark = "navy", light = "lightblue")
#'
get_contrast <- function(..., col, dark = NULL, light = NULL) {
  # Only get theme if we need it
  if (rlang::is_null(dark) || rlang::is_null(light)) {
    # Get current theme
    current_theme <- ggplot2::get_theme()

    # Get text colour from theme
    theme_text <- current_theme$axis.text.x@colour %||%
      current_theme$axis.text.y@colour %||%
      current_theme$axis.text@colour %||%
      current_theme$text@colour %||%
      "black"

    # Get panel background from theme
    theme_panel <- current_theme$panel.background@fill %||%
      "white"

    # Determine which is dark and which is light using is_col_dark
    if (is_col_dark(theme_text)) {
      # Dark text theme (light mode)
      dark <- dark %||% theme_text
      light <- light %||% theme_panel
    } else {
      # Light text theme (dark mode)
      dark <- dark %||% theme_panel
      light <- light %||% theme_text
    }
  }

  # Use is_col_dark to determine which colour to return
  ifelse(!is_col_dark(col), dark, light)
}

#' Check if a colour is dark
#'
#' @description
#' Determines whether a colour is dark by examining its luminance value.
#'
#' @param col A colour value. Can be a hex code, colour name, or any format
#'        accepted by farver. If NULL, returns FALSE.
#'
#' @return TRUE if dark (luminance <= 50) and FALSE otherwise.
#'
#' @noRd
#'
is_col_dark <- function(col) {
  # Handle NULL or missing input
  if (rlang::is_null(col) || length(col) == 0) {
    return(FALSE)
  }

  # Calculate luminance of the colour
  col_luminance <- farver::get_channel(
    colour = col,
    channel = "l",
    space = "hcl"
  )

  # Return TRUE if low luminance
  col_luminance <= 50
}

# axis_line -------------------------------------------------------------------

#' Annotate an axis line
#'
#' Draws a line along an axis edge, with style defaults taken from the
#' `axis.line` element of the set theme. Requires `coord_cartesian(clip = "off")`.
#'
#' @param ... Not used. Forces named arguments.
#' @param position One of `"top"`, `"bottom"`, `"left"`, or `"right"`. Inferred
#'   from `xintercept` or `yintercept` if not provided.
#' @param xintercept For `"left"`/`"right"` axes: float the axis to this x
#'   position in data coordinates instead of the panel edge.
#' @param yintercept For `"top"`/`"bottom"` axes: float the axis to this y
#'   position in data coordinates instead of the panel edge.
#' @param colour Inherits from `axis.line` in the set theme.
#' @param linewidth Inherits from `axis.line` in the set theme. Supports `rel()`.
#' @param linetype Inherits from `axis.line` in the set theme.
#'
#' @return A list of ggplot2 annotation layers.
#' @seealso [axis_ticks()], [axis_text()],
#'   [axis_bracket()], [reference_line()],
#'   [panel_shade()], [sec_axis_text()]
#' @export
axis_line <- function(
    ...,
    position   = NULL,
    xintercept = NULL,
    yintercept = NULL,
    colour     = NULL,
    linewidth  = NULL,
    linetype   = NULL
) {
  rlang::check_dots_empty()

  position <- .infer_position(position, xintercept, yintercept)
  axis     <- if (position %in% c("top", "bottom")) "x" else "y"

  .validate_intercept(axis, position, xintercept, yintercept)

  intercept     <- .resolve_intercept(axis, position, xintercept, yintercept)
  current_theme <- ggplot2::theme_get()

  element_hierarchy <- c(
    paste0("axis.line.", axis, ".", position),
    paste0("axis.line.", axis),
    "axis.line"
  )

  theme_element_blank <- NULL
  for (nm in element_hierarchy) {
    el <- ggplot2::calc_element(nm, current_theme, skip_blank = FALSE)
    if (!is.null(el)) { theme_element_blank <- el; break }
  }
  axis_line_intentionally_blank <- is.null(theme_element_blank) ||
    inherits(theme_element_blank, "element_blank")

  resolved_element <- NULL
  if (!axis_line_intentionally_blank) {
    for (nm in element_hierarchy) {
      el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
      if (!is.null(el) && !inherits(el, "element_blank")) { resolved_element <- el; break }
    }
  }

  if (is.null(colour) && (axis_line_intentionally_blank || is.null(resolved_element$colour))) {
    rlang::warn("The set theme does not define an `axis.line` colour. Defaulting to \"black\".")
  }
  if (is.null(linewidth) && (axis_line_intentionally_blank || is.null(resolved_element$linewidth))) {
    rlang::warn("The set theme does not define an `axis.line` linewidth. Defaulting to `0.5`.")
  }
  if (is.null(resolved_element)) {
    resolved_element <- list(colour = "black", linewidth = 0.5, linetype = 1)
  }

  line_colour    <- colour %||% resolved_element$colour %||% "black"
  line_linewidth <- if (is.null(linewidth)) {
    resolved_element$linewidth %||% 0.5
  } else if (inherits(linewidth, "rel")) {
    as.numeric(linewidth) * (resolved_element$linewidth %||% 0.5)
  } else {
    linewidth
  }
  line_linetype <- linetype %||% resolved_element$linetype %||% 1

  if (axis == "x") {
    list(ggplot2::annotate(
      "segment", x = -Inf, xend = Inf, y = intercept, yend = intercept,
      colour = line_colour, linewidth = line_linewidth, linetype = line_linetype
    ))
  } else {
    list(ggplot2::annotate(
      "segment", x = intercept, xend = intercept, y = -Inf, yend = Inf,
      colour = line_colour, linewidth = line_linewidth, linetype = line_linetype
    ))
  }
}

# guide_sec_axis_text ---------------------------------------------------------------

#' Guide optimised for secondary axis text annotations
#'
#' A wrapper around [ggplot2::guide_axis()] that defaults to using
#' [theme_sec_axis_text()]. This guide is designed to strip away standard axis
#' furniture (like lines and ticks) while preserving text, making it ideal for
#' secondary axes used as margin labels.
#'
#' @param ... Additional arguments passed to [ggplot2::guide_axis()], such as
#'   `title`, `check.overlap`, or `angle`.
#' @param theme A `theme` object to style the guide. Defaults to
#'   [theme_sec_axis_text()], which suppresses ticks and lines.
#'
#' @returns A `guide` object to be used in a scale's `guide` argument or within
#'   [sec_axis_text()].
#'
#' @seealso [sec_axis_text()], [theme_sec_axis_text()]
#'
#' @export
#'
#' @inherit sec_axis_text examples
#'
guide_sec_axis_text <- function(..., theme = theme_sec_axis_text()) {
  ggplot2::guide_axis(
    theme = theme,
    ...
  )
}

# axis_ticks ------------------------------------------------------------------

#' Annotate axis ticks
#'
#' Draws axis ticks at specified break positions, with style defaults taken from
#' the `axis.ticks` element of the set theme. Requires
#' `coord_cartesian(clip = "off")`.
#'
#' @param ... Not used. Forces named arguments.
#' @param position One of `"top"`, `"bottom"`, `"left"`, or `"right"`. Inferred
#'   from `xintercept` or `yintercept` if not provided.
#' @param xintercept For `"left"`/`"right"` axes: float the axis to this x
#'   position in data coordinates instead of the panel edge.
#' @param yintercept For `"top"`/`"bottom"` axes: float the axis to this y
#'   position in data coordinates instead of the panel edge.
#' @param breaks A numeric vector of break positions.
#' @param minor Logical. If `TRUE`, uses minor tick theme defaults. Defaults to
#'   `FALSE`.
#' @param colour Inherits from `axis.ticks` in the set theme.
#' @param linewidth Inherits from `axis.ticks` in the set theme. Supports `rel()`.
#' @param length Total tick length as a grid unit. Supports `rel()`.
#'   Negative values flip the tick direction (inward). Defaults to `rel(1)`
#'   (outward at theme tick length).
#'
#' @return A list of ggplot2 annotation layers.
#' @seealso [axis_line()], [axis_text()],
#'   [axis_bracket()], [reference_line()],
#'   [panel_shade()], [sec_axis_text()]
#' @export
axis_ticks <- function(
    ...,
    position     = NULL,
    xintercept   = NULL,
    yintercept   = NULL,
    breaks,
    minor        = FALSE,
    colour       = NULL,
    linewidth    = NULL,
    length = ggplot2::rel(1)
) {
  rlang::check_dots_empty()

  position <- .infer_position(position, xintercept, yintercept)
  axis     <- if (position %in% c("top", "bottom")) "x" else "y"

  .validate_intercept(axis, position, xintercept, yintercept)

  if (length(breaks) == 0) return(list())

  intercept     <- .resolve_intercept(axis, position, xintercept, yintercept)
  current_theme <- ggplot2::theme_get()

  tick_hierarchy <- if (minor) {
    c(paste0("axis.minor.ticks.", axis, ".", position), paste0("axis.ticks.", axis, ".", position),
      paste0("axis.ticks.", axis), "axis.ticks")
  } else {
    c(paste0("axis.ticks.", axis, ".", position), paste0("axis.ticks.", axis), "axis.ticks")
  }

  resolved_tick_element <- NULL
  tick_intentionally_blank <- FALSE
  for (nm in tick_hierarchy) {
    el_raw <- ggplot2::calc_element(nm, current_theme, skip_blank = FALSE)
    if (!is.null(el_raw)) {
      if (inherits(el_raw, "element_blank")) { tick_intentionally_blank <- TRUE; break }
      el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
      if (!is.null(el) && !inherits(el, "element_blank")) { resolved_tick_element <- el; break }
    }
  }

  if (is.null(colour) && (tick_intentionally_blank || is.null(resolved_tick_element$colour))) {
    rlang::warn("The set theme does not define an `axis.ticks` colour. Defaulting to \"black\".")
  }
  if (is.null(linewidth) && (tick_intentionally_blank || is.null(resolved_tick_element$linewidth))) {
    rlang::warn("The set theme does not define an `axis.ticks` linewidth. Defaulting to `0.5`.")
  }
  if (is.null(resolved_tick_element)) {
    resolved_tick_element <- list(colour = "black", linewidth = 0.5)
  }

  length_hierarchy <- if (minor) {
    c(paste0("axis.minor.ticks.length.", axis, ".", position), paste0("axis.minor.ticks.length.", axis),
      "axis.minor.ticks.length", paste0("axis.ticks.length.", axis, ".", position),
      paste0("axis.ticks.length.", axis), "axis.ticks.length")
  } else {
    c(paste0("axis.ticks.length.", axis, ".", position), paste0("axis.ticks.length.", axis), "axis.ticks.length")
  }

  resolved_length_element <- NULL
  for (nm in length_hierarchy) {
    el <- ggplot2::calc_element(nm, current_theme, skip_blank = TRUE)
    if (!is.null(el) && !inherits(el, "element_blank")) { resolved_length_element <- el; break }
  }

  tick_colour    <- colour %||% resolved_tick_element$colour %||% "black"
  tick_linewidth <- if (is.null(linewidth)) {
    resolved_tick_element$linewidth %||% 0.5
  } else if (inherits(linewidth, "rel")) {
    as.numeric(linewidth) * (resolved_tick_element$linewidth %||% 0.5)
  } else {
    linewidth
  }

  calculate_theme_length <- function() {
    tl <- resolved_length_element
    if (is.null(tl)) {
      return(grid::unit((if (minor) 0.375 else 0.5) * (current_theme$text$size %||% 11), "pt"))
    } else if (inherits(tl, "rel")) {
      spacing_pts <- as.numeric(grid::convertUnit(current_theme$spacing %||% grid::unit(5.5, "pt"), "pt"))
      return(grid::unit(as.numeric(tl) * spacing_pts, "pt"))
    } else if (!inherits(tl, "unit")) {
      return(grid::unit(
        if (is.numeric(tl)) tl else (if (minor) 0.375 else 0.5) * (current_theme$text$size %||% 11), "pt"
      ))
    } else {
      return(tl)
    }
  }

  flip_direction <- FALSE
  if (inherits(length, "rel")) {
    rel_value      <- as.numeric(length)
    default_pts    <- as.numeric(grid::convertUnit(calculate_theme_length(), "pt"))
    length   <- grid::unit(abs(rel_value) * default_pts, "pt")
    flip_direction <- rel_value < 0
  } else if (inherits(length, "unit")) {
    tick_pts       <- as.numeric(grid::convertUnit(length, "pt"))
    length   <- grid::unit(abs(tick_pts), "pt")
    flip_direction <- tick_pts < 0
  } else if (is.numeric(length)) {
    flip_direction <- length < 0
    length   <- grid::unit(abs(length), "pt")
  }

  gp <- ggplot2::gg_par(col = tick_colour, stroke = tick_linewidth, lineend = "butt")

  lapply(breaks, \(break_val) {
    tick_grob <- if (position == "bottom") {
      grid::segmentsGrob(
        x0 = grid::unit(0.5, "npc"), x1 = grid::unit(0.5, "npc"),
        y0 = grid::unit(0, "npc"),
        y1 = if (flip_direction) grid::unit(0, "npc") + length
        else                grid::unit(0, "npc") - length,
        gp = gp
      )
    } else if (position == "top") {
      grid::segmentsGrob(
        x0 = grid::unit(0.5, "npc"), x1 = grid::unit(0.5, "npc"),
        y0 = grid::unit(1, "npc"),
        y1 = if (flip_direction) grid::unit(1, "npc") - length
        else                grid::unit(1, "npc") + length,
        gp = gp
      )
    } else if (position == "left") {
      grid::segmentsGrob(
        x0 = grid::unit(0, "npc"),
        x1 = if (flip_direction) grid::unit(0, "npc") + length
        else                grid::unit(0, "npc") - length,
        y0 = grid::unit(0.5, "npc"), y1 = grid::unit(0.5, "npc"),
        gp = gp
      )
    } else {
      grid::segmentsGrob(
        x0 = grid::unit(1, "npc"),
        x1 = if (flip_direction) grid::unit(1, "npc") - length
        else                grid::unit(1, "npc") + length,
        y0 = grid::unit(0.5, "npc"), y1 = grid::unit(0.5, "npc"),
        gp = gp
      )
    }

    annotation_position <- if (axis == "x") {
      list(xmin = break_val, xmax = break_val, ymin = intercept, ymax = intercept)
    } else {
      list(xmin = intercept, xmax = intercept, ymin = break_val, ymax = break_val)
    }

    rlang::exec(ggplot2::annotation_custom, grob = tick_grob, !!!annotation_position)
  })
}

# theme_sec_axis_text ------------------------------------------------------------------

#' Theme adjustments optimised for secondary axis text annotations
#'
#' @param axis Character. "x", "y", or NULL (defaults to both).
#' @param axis_ticks_to Action for ticks: "transparent", "blank", or "keep".
#' @param axis_line_to Action for lines: "transparent", "blank", or "keep".
#' @param axis_text_to Action for text: "transparent", "blank", or "keep".
#' @param axis_title_to Action for titles: "transparent", "blank", or "keep".
#'
#' @returns A ggplot2 theme object.
#' @export
#'
#' @seealso [sec_axis_text()], [guide_sec_axis_text()]
#'
#' @seealso [axis_ticks()], [axis_line()],
#' [axis_text()], [reference_line()]
#'
theme_sec_axis_text <- function(
    axis = NULL,
    axis_ticks_to = "transparent",
    axis_line_to = "transparent",
    axis_text_to = "keep",
    axis_title_to = "keep"
) {

  # Define which sides of the plot to target based on the axis argument
  if (is.null(axis)) {
    targets <- c("x.top", "x.bottom", "y.left", "y.right")
  } else if (axis == "x") {
    targets <- c("x.top", "x.bottom")
  } else if (axis == "y") {
    targets <- c("y.left", "y.right")
  } else {
    stop("Invalid axis argument. Use 'x', 'y', or NULL.")
  }

  # Helper function to return the correct ggplot2 element
  get_element <- function(action) {
    if (action == "keep") return(NULL)
    if (action == "blank") return(ggplot2::element_blank())
    if (action == "transparent") return(ggplot2::element_line(colour = "transparent"))
    return(NULL)
  }

  # Initialize an empty list to store theme arguments
  theme_args <- list()

  # Loop through targets and assign the specified elements
  for (side in targets) {

    # Ticks logic
    tick_val <- get_element(axis_ticks_to)
    if (!is.null(tick_val)) theme_args[[paste0("axis.ticks.", side)]] <- tick_val

    # Line logic
    line_val <- get_element(axis_line_to)
    if (!is.null(line_val)) theme_args[[paste0("axis.line.", side)]] <- line_val

    # Text logic
    text_val <- get_element(axis_text_to)
    if (!is.null(text_val)) theme_args[[paste0("axis.text.", side)]] <- text_val

    # Title logic
    title_val <- get_element(axis_title_to)
    if (!is.null(title_val)) theme_args[[paste0("axis.title.", side)]] <- title_val
  }

  # Return the combined theme
  do.call(ggplot2::theme, theme_args)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/axis-text.R
\name{axis_text}
\alias{axis_text}
\title{Annotate axis text}
\usage{
axis_text(
  ...,
  position = NULL,
  xintercept = NULL,
  yintercept = NULL,
  breaks,
  labels = NULL,
  colour = NULL,
  size = NULL,
  family = NULL,
  hjust = NULL,
  vjust = NULL,
  angle = 0,
  length = ggplot2::rel(1)
)
}
\arguments{
\item{...}{Not used. Forces named arguments.}

\item{position}{One of \code{"top"}, \code{"bottom"}, \code{"left"}, or \code{"right"}. Inferred
from \code{xintercept} or \code{yintercept} if not provided.}

\item{xintercept}{For \code{"left"}/\code{"right"} axes: float the axis to this x
position in data coordinates instead of the panel edge.}

\item{yintercept}{For \code{"top"}/\code{"bottom"} axes: float the axis to this y
position in data coordinates instead of the panel edge.}

\item{breaks}{A numeric vector of break positions.}

\item{labels}{One of:
\itemize{
\item \code{NULL} (default) to use break values as labels
\item A character vector the same length as \code{breaks}
\item A function taking break values and returning labels
}}

\item{colour}{Inherits from \code{axis.text} in the set theme.}

\item{size}{Inherits from \code{axis.text} in the set theme.}

\item{family}{Inherits from \code{axis.text} in the set theme.}

\item{hjust, vjust}{Justification. Auto-calculated from \code{position} if \code{NULL}.}

\item{angle}{Text rotation angle. Defaults to \code{0}.}

\item{length}{Offset from the axis edge including tick length and margin.
Supports \code{rel()}. Negative values place labels inside the panel. Defaults
to \code{rel(1)} (theme tick length + text margin).}
}
\value{
A list of ggplot2 annotation layers.
}
\description{
Draws text labels at specified break positions along an axis, with style
defaults taken from the \code{axis.text} element of the set theme. Requires
\code{coord_cartesian(clip = "off")}.
}
\examples{
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #'geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))

}
\seealso{
\code{\link[=axis_line]{axis_line()}}, \code{\link[=axis_ticks]{axis_ticks()}},
\code{\link[=axis_bracket]{axis_bracket()}}, \code{\link[=reference_line]{reference_line()}},
\code{\link[=panel_shade]{panel_shade()}}, \code{\link[=sec_axis_text]{sec_axis_text()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-sec-axis-text.R
\name{guide_sec_axis_text}
\alias{guide_sec_axis_text}
\title{Guide optimised for secondary axis text annotations}
\usage{
guide_sec_axis_text(..., theme = theme_sec_axis_text())
}
\arguments{
\item{...}{Additional arguments passed to \code{\link[ggplot2:guide_axis]{ggplot2::guide_axis()}}, such as
\code{title}, \code{check.overlap}, or \code{angle}.}

\item{theme}{A \code{theme} object to style the guide. Defaults to
\code{\link[=theme_sec_axis_text]{theme_sec_axis_text()}}, which suppresses ticks and lines.}
}
\value{
A \code{guide} object to be used in a scale's \code{guide} argument or within
\code{\link[=sec_axis_text]{sec_axis_text()}}.
}
\description{
A wrapper around \code{\link[ggplot2:guide_axis]{ggplot2::guide_axis()}} that defaults to using
\code{\link[=theme_sec_axis_text]{theme_sec_axis_text()}}. This guide is designed to strip away standard axis
furniture (like lines and ticks) while preserving text, making it ideal for
secondary axes used as margin labels.
}
\examples{
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #'geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))

}
\seealso{
\code{\link[=sec_axis_text]{sec_axis_text()}}, \code{\link[=theme_sec_axis_text]{theme_sec_axis_text()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-sec-axis-text.R
\name{theme_sec_axis_text}
\alias{theme_sec_axis_text}
\title{Theme adjustments optimised for secondary axis text annotations}
\usage{
theme_sec_axis_text(
  axis = NULL,
  axis_ticks_to = "transparent",
  axis_line_to = "transparent",
  axis_text_to = "keep",
  axis_title_to = "keep"
)
}
\arguments{
\item{axis}{Character. "x", "y", or NULL (defaults to both).}

\item{axis_ticks_to}{Action for ticks: "transparent", "blank", or "keep".}

\item{axis_line_to}{Action for lines: "transparent", "blank", or "keep".}

\item{axis_text_to}{Action for text: "transparent", "blank", or "keep".}

\item{axis_title_to}{Action for titles: "transparent", "blank", or "keep".}
}
\value{
A ggplot2 theme object.
}
\description{
Theme adjustments optimised for secondary axis text annotations
}
\seealso{
\code{\link[=sec_axis_text]{sec_axis_text()}}, \code{\link[=guide_sec_axis_text]{guide_sec_axis_text()}}

\code{\link[=axis_ticks]{axis_ticks()}}, \code{\link[=axis_line]{axis_line()}},
\code{\link[=axis_text]{axis_text()}}, \code{\link[=reference_line]{reference_line()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggscribe-package.R
\docType{package}
\name{ggscribe-package}
\alias{ggscribe}
\alias{ggscribe-package}
\title{ggscribe: Publication-Quality 'ggplot2' Annotation}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Annotation helper functions for publication-quality 'ggplot2' visualisation. These functions make it easier to annotate plots in a way that stays consistent with the set theme.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/davidhodge931/ggscribe}
  \item \url{https://davidhodge931.github.io/ggscribe/}
  \item Report bugs at \url{https://github.com/davidhodge931/ggscribe/issues}
}

}
\author{
\strong{Maintainer}: David Hodge \email{davidhodge931@gmail.com} (\href{https://orcid.org/0000-0002-3868-7501}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/axis-line.R
\name{axis_line}
\alias{axis_line}
\title{Annotate an axis line}
\usage{
axis_line(
  ...,
  position = NULL,
  xintercept = NULL,
  yintercept = NULL,
  colour = NULL,
  linewidth = NULL,
  linetype = NULL
)
}
\arguments{
\item{...}{Not used. Forces named arguments.}

\item{position}{One of \code{"top"}, \code{"bottom"}, \code{"left"}, or \code{"right"}. Inferred
from \code{xintercept} or \code{yintercept} if not provided.}

\item{xintercept}{For \code{"left"}/\code{"right"} axes: float the axis to this x
position in data coordinates instead of the panel edge.}

\item{yintercept}{For \code{"top"}/\code{"bottom"} axes: float the axis to this y
position in data coordinates instead of the panel edge.}

\item{colour}{Inherits from \code{axis.line} in the set theme.}

\item{linewidth}{Inherits from \code{axis.line} in the set theme. Supports \code{rel()}.}

\item{linetype}{Inherits from \code{axis.line} in the set theme.}
}
\value{
A list of ggplot2 annotation layers.
}
\description{
Draws a line along an axis edge, with style defaults taken from the
\code{axis.line} element of the set theme. Requires \code{coord_cartesian(clip = "off")}.
}
\seealso{
\code{\link[=axis_ticks]{axis_ticks()}}, \code{\link[=axis_text]{axis_text()}},
\code{\link[=axis_bracket]{axis_bracket()}}, \code{\link[=reference_line]{reference_line()}},
\code{\link[=panel_shade]{panel_shade()}}, \code{\link[=sec_axis_text]{sec_axis_text()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reference-line.R
\name{reference_line}
\alias{reference_line}
\title{Annotate a reference line}
\usage{
reference_line(
  ...,
  xintercept = NULL,
  yintercept = NULL,
  colour = NULL,
  linewidth = NULL,
  linetype = "dashed"
)
}
\arguments{
\item{...}{Not used. Forces named arguments.}

\item{xintercept}{Draw a vertical reference line at this x position.}

\item{yintercept}{Draw a horizontal reference line at this y position.}

\item{colour}{Inherits from \code{axis.line} in the set theme.}

\item{linewidth}{Inherits from \code{axis.line} in the set theme. Supports \code{rel()}.}

\item{linetype}{Defaults to \code{"dashed"}.}
}
\value{
A list of ggplot2 annotation layers.
}
\description{
Draws a reference line within the panel, with style defaults taken from the
\code{axis.line} element of the set theme.
}
\examples{
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #'geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))

}
\seealso{
\code{\link[=axis_line]{axis_line()}}, \code{\link[=axis_ticks]{axis_ticks()}},
\code{\link[=axis_text]{axis_text()}}, \code{\link[=axis_bracket]{axis_bracket()}}, \code{\link[=panel_shade]{panel_shade()}},
\code{\link[=sec_axis_text]{sec_axis_text()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aes-contrast.R
\name{aes_contrast}
\alias{aes_contrast}
\title{A mapped aesthetic for text colour on fill}
\usage{
aes_contrast(..., dark = NULL, light = NULL, aesthetic = "colour")
}
\arguments{
\item{...}{Require named arguments (and support trailing commas).}

\item{dark}{A dark colour. If NULL, derived from theme text or panel background.}

\item{light}{A light colour. If NULL, derived from theme text or panel background.}

\item{aesthetic}{The aesthetic to be modified for contrast. Either "colour" (default) or "fill".}
}
\value{
A ggplot2 aesthetic in \link[ggplot2:aes]{ggplot2::aes}.
}
\description{
Modifies a mapped colour (or fill) aesthetic for contrast against the fill (or colour) aesthetic.

Function can be spliced into \link[ggplot2:aes]{ggplot2::aes} with \link[rlang:splice-operator]{rlang::!!!}.
}
\examples{
library(ggplot2)
library(dplyr)
library(stringr)

set_theme(
 ggrefine::theme_light(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
 )
)

ggwidth::set_equiwidth(equiwidth = 1.75)

mtcars |>
  count(cyl, am) |>
  mutate(
    am = if_else(am == 0, "Automatic", "Manual"),
    cyl = as.factor(cyl)
  ) |>
  ggplot(aes(x = am, y = n, colour = cyl, fill = cyl, label = n)) +
  geom_col(
    position = position_dodge2(preserve = "single", padding = 0.05),
    width = ggwidth::get_width(n = 2, n_dodge = 3),
  ) +
  scale_fill_discrete(palette = jumble::jumble) +
  scale_colour_discrete(palette = blends::multiply(jumble::jumble)) +
  geom_text(
    mapping = ggscribe::aes_contrast(), # or aes(!!!ggscribe::aes_contrast()),
    position = position_dodge2(
      width = ggwidth::get_width(n = 2, n_dodge = 3),
      padding = 0.05,
      preserve = "single"),
    vjust = 1.33,
    show.legend = FALSE,
  ) +
  scale_y_continuous(expand = expansion(c(0, 0.05))) +
  ggrefine::modern(x_type = "discrete")

mtcars |>
  count(cyl, am) |>
  mutate(
    am = if_else(am == 0, "automatic", "manual"),
    am = stringr::str_to_sentence(am),
    cyl = as.factor(cyl)
  ) |>
  ggplot(aes(y = am, x = n, colour = cyl, fill = cyl, label = n)) +
  geom_col(
    position = position_dodge2(preserve = "single", padding = 0.05),
    width = ggwidth::get_width(n = 2, n_dodge = 3, orientation = "y"),
  ) +
  scale_fill_discrete(palette = jumble::jumble) +
  scale_colour_discrete(palette = blends::multiply(jumble::jumble)) +
  geom_text(
    mapping = ggscribe::aes_contrast(), # or aes(!!!ggscribe::aes_contrast()),
    position = position_dodge2(
      width = ggwidth::get_width(n = 2, n_dodge = 3, orientation = "y"),
      preserve = "single",
      padding = 0.05,
    ),
    hjust = 1.25,
    show.legend = FALSE,
  ) +
  scale_x_continuous(expand = expansion(c(0, 0.05))) +
  ggrefine::modern(y_type = "discrete")

}
\seealso{
\code{\link[rlang]{splice}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sec-axis-text.R
\name{sec_axis_text}
\alias{sec_axis_text}
\title{Secondary axis optimised for text annotations}
\usage{
sec_axis_text(
  breaks = ggplot2::waiver(),
  labels = ggplot2::derive(),
  name = NULL,
  guide = ggplot2::guide_axis(theme = theme_sec_axis_text()),
  ...
)
}
\arguments{
\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{\link[ggplot2:waiver]{ggplot2::waiver()}} (default) to inherit breaks from the primary axis
\item A numeric vector of break positions
\item A function that takes the scale limits as input and returns break
positions (e.g. \verb{\\(x) mean(c(x[2], 32))})
}}

\item{labels}{One of:
\itemize{
\item \code{\link[ggplot2:sec_axis]{ggplot2::derive()}} (default) to derive labels from \code{breaks}
\item A character vector of labels, the same length as \code{breaks}
\item A function that takes break positions as input and returns labels
}}

\item{name}{The name of the secondary axis. Use \code{\link[ggplot2:waiver]{ggplot2::waiver()}} to
derive the name from the primary axis, or \code{NULL} (default) for no name.}

\item{guide}{A guide object used to render the axis. Defaults to
\code{\link[=guide_sec_axis_text]{guide_sec_axis_text()}}, which uses \code{\link[=theme_sec_axis_text]{theme_sec_axis_text()}} to
make transparent ticks and lines by default.}

\item{...}{Additional arguments passed to \code{\link[ggplot2:sec_axis]{ggplot2::dup_axis()}}.}
}
\value{
A \code{AxisSecondary} object for use in the \code{sec.axis} argument of
\code{scale_x_continuous()} or \code{scale_y_continuous()}.
}
\description{
Secondary axis optimised for text annotations
}
\examples{
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #'geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))

}
\seealso{
\code{\link[=guide_sec_axis_text]{guide_sec_axis_text()}}, \code{\link[=theme_sec_axis_text]{theme_sec_axis_text()}}, \code{\link[=axis_text]{axis_text()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/panel-shade.R
\name{panel_shade}
\alias{panel_shade}
\title{Annotate a shaded panel region}
\usage{
panel_shade(
  ...,
  xmin = -Inf,
  xmax = Inf,
  ymin = -Inf,
  ymax = Inf,
  fill = "#878580",
  alpha = 0.25,
  colour = "transparent",
  linewidth = NULL,
  linetype = NULL
)
}
\arguments{
\item{...}{Not used. Allows trailing commas and named-argument style calls.}

\item{xmin, xmax}{Left and right edges of the rectangle. Defaults to \code{-Inf}
and \code{Inf}. Use \code{I()} for normalized coordinates (0-1).}

\item{ymin, ymax}{Bottom and top edges of the rectangle. Defaults to \code{-Inf}
and \code{Inf}. Use \code{I()} for normalized coordinates (0-1).}

\item{fill}{Fill colour. Defaults to a neutral grey.}

\item{alpha}{Opacity of the rectangle. Defaults to \code{0.25}.}

\item{colour}{Border colour. Defaults to \code{"transparent"}.}

\item{linewidth}{Inherits from \code{panel.border} in the set theme. Supports
\code{rel()}.}

\item{linetype}{Border linetype. Defaults to \code{1}.}
}
\value{
A list containing an annotation layer.
}
\description{
Draws a filled rectangle over the panel with colour defaults taken from the
set theme. Defaults to a subtle overlay across the full panel, with the fill
automatically adapting to light or dark panel backgrounds. Should be placed
before geom layers.
}
\examples{
library(ggplot2)
library(dplyr)

set_theme(
  ggrefine::theme_grey(
    panel_heights = rep(unit(50, "mm"), 100),
    panel_widths = rep(unit(75, "mm"), 100),
  )
)

mtcars |>
  ggplot(aes(x = wt, y = mpg, colour = as.factor(gear), fill = as.factor(gear))) +
  scale_colour_discrete(palette = blends::multiply(get_theme()$palette.colour.discrete)) +
  #clip = "off" is required for axis_text, axis_ticks and axis_bracket
  coord_cartesian(clip = "off") +
  #reference lines and shade
  ggscribe::reference_line(xintercept = 2.4) +
  ggscribe::reference_line(yintercept = 12)  +
  ggscribe::panel_shade(
    xmin = 4,
    xmax = 5,
  ) +
  #top axis
  scale_x_continuous(
    sec.axis = ggscribe::sec_axis_text(
      breaks = c(mean(c(4, 5))),
      labels = c("Range"),
      guide = ggscribe::guide_sec_axis_text(
        angle = 90,
      )
    )
  ) +
  ggscribe::axis_bracket(
    position = "top",
    breaks = c(4, 5),
  ) +
  ggscribe::axis_text(
    position = "top",
    breaks = c(2.4),
    labels = c("Threshold"),
  ) +
  #right axis
  ggscribe::axis_text(
    position = "right",
    breaks = 12,
    labels = "Threshold",
  ) +
  #'geom
  geom_point() +
  #annotations fit plot
  theme(plot.background = element_rect(colour = "grey92"))

}
